#include "StateNameModule.hpp"

StateNameModuleImpl::StateNameModuleImpl(const std::string & definition)
{
  std::regex regex("(?:(?:\\s|\\t)*)States\\{(.*)\\}(?:(?:\\s|\\t)*)Out\\{(.*)\\}(?:(?:\\s|\\t)*)");
  if (!util::doIfNameMatch(regex, definition, [this,&definition](auto sm)
        {
          try
          {
            auto states = util::split(sm.str(1), ' ');
            outSize = std::stoi(sm.str(2));

            for (auto & state : states)
              state2index.emplace(state, state2index.size());
          } catch (std::exception & e) {util::myThrow(fmt::format("{} in '{}'",e.what(),definition));}
        }))
    util::myThrow(fmt::format("invalid definition '{}'", definition));
}

torch::Tensor StateNameModuleImpl::forward(torch::Tensor input)
{
  return embeddings(input.narrow(1,firstInputIndex,1).squeeze(1));
}

std::size_t StateNameModuleImpl::getOutputSize()
{
  return outSize;
}

std::size_t StateNameModuleImpl::getInputSize()
{
  return 1;
}

void StateNameModuleImpl::addToContext(std::vector<std::vector<long>> & context, Dict & dict, const Config & config) const
{
  for (auto & contextElement : context)
    contextElement.emplace_back(state2index.at(config.getState()));
}

void StateNameModuleImpl::registerEmbeddings(std::size_t)
{
  embeddings = register_module("embeddings", torch::nn::Embedding(state2index.size(), outSize));
}

